@@ -4,6 +4,8 @@
 from datetime import datetime
 import sdc_helpers.utils as utils
 import boto3
+from botocore.exceptions import ClientError as botocore_clienterror
+from sagemaker.utils import name_from_base
 from sdc_engine_helpers.date_utils import DateUtils
 
 class CampaignStateManager:
@@ -19,6 +21,20 @@ class CampaignStateManager:
             - update status of campaign in database
 
     """
+    resource_tags = [
+        {
+            'Key': 'environment',
+            'Value': 'dev'
+        },
+        {
+            'Key': 'engine',
+            'Value': 'alice'
+        },
+        {
+            'Key': 'project',
+            'Value': 'recommend'
+        },
+    ]
     date_utils = DateUtils()
     # vendor
     vendor_solution_arn_path = 'TrainingJobName'
@@ -59,28 +75,64 @@ class CampaignStateManager:
         self.sagemaker = boto3.client('sagemaker')
 
    # pylint: disable=invalid-name
-    def sagemaker_create_model(self, TrainingJobName: str):
-        """Create model from a trained model job using Sagemaker API"""
-        model_name = TrainingJobName
+    def sagemaker_create_model(self, training_job_name: str) -> str:
+        """
+            Create model from a trained model job using Sagemaker API.
+            if model exists, creation is skipped.
+
+            note:
+                - Model name is the training job name
+
+            Args:
+                TrainingJobName (str): Naming of sagemaker training job
+
+            Returns:
+                str: model name
+        """
+        model_name = training_job_name
 
         training_job_config = self.sagemaker.describe_training_job(
-            TrainingJobName=TrainingJobName
+            TrainingJobName=training_job_name
         )
 
         primary_container = {
             'Image': training_job_config['AlgorithmSpecification']['TrainingImage'],
+            'Mode': 'SingleModel',
+            'Environment': training_job_config['HyperParameters'],
             'ModelDataUrl': training_job_config['ModelArtifacts']['S3ModelArtifacts']
         }
 
-        _ = self.sagemaker.create_model(
-            ModelName=model_name,
-            ExecutionRoleArn=training_job_config['RoleArn'],
-            PrimaryContainer=primary_container
-        )
+        model_tags = self.resource_tags
+        model_tags.append({
+            'Key': 'status',
+            'Value': 'active'
+        })
+        try:
+            _ = self.sagemaker.create_model(
+                ModelName=model_name,
+                ExecutionRoleArn=training_job_config['RoleArn'],
+                PrimaryContainer=primary_container,
+                Tags=model_tags
+            )
+
+        except botocore_clienterror as exception:
+            if exception.response['Error']['Code'] == 'ValidationException':
+                # handle no
+                if 'Cannot create already existing model' in exception.response['Error']['Message']:
+                    print((
+                        "This Model already exists, "
+                        "with name = {}. Using the already created model."
+                        .format(model_name)
+                    ))
+                else:
+                    raise botocore_clienterror(
+                        error_response=exception.operation_name,
+                        operation_name=exception.response
+                    )
 
         return model_name
 
-    def sagemaker_update_campaign(self, EndpointName: str, TrainingJobName: str):
+    def sagemaker_update_campaign(self, endpoint_name: str, training_job_name: str):
         """
             (Call vendor API) Creates a endpoint config
             and updates endpoint with this config
@@ -92,20 +144,27 @@ class CampaignStateManager:
             Returns:
                 dict: contains names of new created resources
         """
-        endpoint_config_name = "{}-{}".format("Campaign", TrainingJobName)
+        # each endpoint configuration needs unique name
+        endpoint_config_name = name_from_base(
+            endpoint_name,
+            max_length=63,
+            short=False
+        )
 
         endpoint = self.sagemaker.describe_endpoint(
-            EndpointName=EndpointName)
+            EndpointName=endpoint_name
+        )
 
-        ModelName = self.sagemaker_create_model(TrainingJobName=TrainingJobName)
+        model_name = self.sagemaker_create_model(training_job_name=training_job_name)
 
         previous_endpoint_config = self.sagemaker.describe_endpoint_config(
-            EndpointConfigName=endpoint['EndpointConfigName'])
+            EndpointConfigName=endpoint['EndpointConfigName']
+        )
 
         production_varients = previous_endpoint_config['ProductionVariants']
         # update the model name in production varients
-        production_varients[0]['ModelName'] = ModelName
-        production_varients[0]['VariantName'] = ModelName
+        production_varients[0]['ModelName'] = model_name
+        production_varients[0]['VariantName'] = model_name
 
         _ = self.sagemaker.create_endpoint_config(
             EndpointConfigName=endpoint_config_name,
@@ -113,14 +172,14 @@ class CampaignStateManager:
         )
 
         _ = self.sagemaker.update_endpoint(
-            EndpointName=EndpointName,
+            EndpointName=endpoint_name,
             EndpointConfigName=endpoint_config_name
         )
 
         return {
-            'EndpointName': EndpointName,
-            'TrainingJobName':TrainingJobName,
-            'ModelName': ModelName
+            'EndpointName': endpoint_name,
+            'TrainingJobName': training_job_name,
+            'ModelName': model_name
         }
 
     def get_vendor_solution_version_arn(self, vendor_solution: dict):
@@ -207,11 +266,12 @@ class CampaignStateManager:
             'should_update': False,
             'should_refresh_status': False
         }
+
         # get vendor campaign
         vendor_campaign = self.sagemaker.describe_endpoint(
             EndpointName=campaign.get('arn')
         )
-
+        # get vendor solution
         vendor_solution = self.sagemaker.describe_training_job(
             TrainingJobName=solution.get('arn')
         )
@@ -221,6 +281,7 @@ class CampaignStateManager:
         )
 
         state['should_update'] = self.should_update(
+            campaign=campaign,
             vendor_campaign=vendor_campaign,
             vendor_solution=vendor_solution
         )
@@ -238,15 +299,28 @@ class CampaignStateManager:
                 vendor_campaign=vendor_campaign
             )
 
+            # get solution_version_arn from vendor campaign
+            vendor_campaign_solution_version_arn = self.get_vendor_campaign_solution_version_arn(
+                vendor_campaign=vendor_campaign
+            )
+
             if vendor_campaign_status != status:
+                # get current refresh time, for keeping track
+                current_time_at_update = datetime.strftime(
+                    datetime.now(),
+                    self.date_utils.get_mysql_date_format()
+                )
                 state['should_refresh_status'] = True
                 state['new_status'] = vendor_campaign_status
+                state['latest_solution_version_arn'] = vendor_campaign_solution_version_arn
+                state['last_refreshed_at'] = current_time_at_update
 
         return state
 
     def should_update(
             self,
             *,
+            campaign: dict,
             vendor_campaign: dict,
             vendor_solution: dict
     ) -> bool:
@@ -259,7 +333,6 @@ class CampaignStateManager:
 
             returns:
                 result (bool): Should update campaign
-
         """
 
         # get solution_version_arn from vendor campaign
@@ -267,6 +340,9 @@ class CampaignStateManager:
             vendor_campaign=vendor_campaign
         )
 
+        # get solution version arn in database
+        database_campaign_solution_version_arn = campaign['solution_version_arn']
+
         # get solution arn from vendor solution
         vendor_solution_version_arn = self.get_vendor_solution_version_arn(
             vendor_solution=vendor_solution
@@ -282,7 +358,14 @@ class CampaignStateManager:
             vendor_solution=vendor_solution
         )
 
-        # Don't update dataset if its on the latest solution version
+        # don't update if mismatch in vendor and database solution version arns
+        if (
+                database_campaign_solution_version_arn !=
+                vendor_campaign_solution_version_arn
+        ):
+            return False
+
+        # Don't update campaign if its on the latest solution version
         if (
                 vendor_solution_version_arn ==
                 vendor_campaign_solution_version_arn
@@ -332,8 +415,8 @@ class CampaignStateManager:
 
         # run vendor campaign update workflow
         response = self.sagemaker_update_campaign(
-            EndpointName=vendor_campaign.get('EndpointName'),
-            TrainingJobName=vendor_solution.get('TrainingJobName')
+            endpoint_name=vendor_campaign.get('EndpointName'),
+            training_job_name=vendor_solution.get('TrainingJobName')
         )
         new_campaign['arn'] = response.get('EndpointName')
         new_campaign['solution_version_arn'] = vendor_solution.get('TrainingJobName')
@@ -356,8 +439,9 @@ class CampaignStateManager:
         # deprecate this going forward
         next_update_time = new_campaign.get('next_update_time', None)
         if next_update_time is not None:
+            # next update time not implemented
             new_campaign.update({
-                'next_update_time': next_update_time
+                'next_update_time': None
             })
 
         return new_campaign
