@@ -10,6 +10,9 @@
         https://boto3.amazonaws.com/v1/documentation/api/latest/guide/events.html
 """
 import boto3
+from sqlalchemy.exc import InternalError
+from botocore.exceptions import ClientError as botocore_clienterror
+from sdc_engine_helpers import decorators
 from sdc_engine_helpers.sagemaker.sagemaker_query_helpers import \
     SagemakerEngineHelpers
 from sdc_engine_helpers.sagemaker.maintenance.campaign_state_manager import \
@@ -18,6 +21,7 @@ from sdc_engine_helpers.sagemaker.maintenance.solution_state_manager import \
     SolutionStateManager
 from sdc_engine_helpers.sagemaker.maintenance.dataset_state_manager import \
     DatasetStateManager
+from sdc_engine_helpers.models.subscription import Subscription
 from sdc_engine_helpers.date_utils import DateUtils
 
 class MaintenanceManager:
@@ -76,6 +80,14 @@ class MaintenanceManager:
 
         for client in self.query_helper.get_clients():
 
+            # get subscription for engine
+            subscription = self.query_helper.get_subscription(
+                client_id=client.id,
+                service_id=self.service.id,
+                from_cache=False,
+                for_update=True
+            )
+
             solution_updated = self.perform_solution_maintenance(
                 client=client,
                 engine_slug=self.engine_slug
@@ -94,26 +106,24 @@ class MaintenanceManager:
             if any([solution_updated, campaign_updated, datasets_updated]):
                 # when any of the above have been updated
                 database_session_updated = self.perform_session_update(
-                    client_id=client.id
+                    subscription=subscription
                 )
 
         return database_session_updated
 
-    def perform_session_update(self, client_id: id):
+    @decorators.retry_handler(
+        exceptions=(Exception, InternalError,),
+        total_tries=4,
+        initial_wait=0.5,
+        backoff_factor=2
+    )
+    def perform_session_update(self, subscription: Subscription) -> bool:
         """
             Perform the session update in the database to commit update to database
 
             Returns:
                 updated (bool): whether the database session was updated
         """
-
-        subscription = self.query_helper.get_subscription(
-            client_id=client_id,
-            service_id=self.service.id,
-            from_cache=False,
-            for_update=True
-        )
-
         self.query_helper.update(
             model=subscription
         )
@@ -145,8 +155,16 @@ class MaintenanceManager:
             for_update=True
         )
 
+        if datasets is None:
+            return datasets_updated
+
         for _, dataset in enumerate(datasets):
 
+            label = dataset.get('label', None)
+            if label is None:
+                # skip this dataset
+                continue
+
             new_dataset = {}
 
             # get solution from db
@@ -155,6 +173,7 @@ class MaintenanceManager:
                 service_id=self.service.id,
                 engine_slug=engine_slug,
                 from_cache=False,
+                for_update=True,
                 recipe=dataset['recipe']
             )
 
@@ -166,21 +185,38 @@ class MaintenanceManager:
 
             if curr_state.get('should_update') is True:
 
-                new_dataset = self.dataset_state_manager.update_state(
-                    dataset=dataset,
-                    campaign=campaign
+                print(
+                    'Updating dataset version for client: {client_name}'
+                    .format(client_name=client.name)
                 )
+                try:
+                    new_dataset = self.dataset_state_manager.update_state(
+                        dataset=dataset,
+                        campaign=campaign
+                    )
 
-                # update dataset
-                self.query_helper.update_dataset(
-                    client_id=client.id,
-                    service_id=self.service.id,
-                    engine_slug=engine_slug,
-                    label=dataset['label'],
-                    config=new_dataset
-                )
+                    # update dataset
+                    self.query_helper.update_dataset(
+                        client_id=client.id,
+                        service_id=self.service.id,
+                        engine_slug=engine_slug,
+                        label=dataset['label'],
+                        config=new_dataset
+                    )
 
-                datasets_updated = True
+                    dataset_label = dataset['label']
+                    print(
+                        'Successfully updated dataset version: {dataset_label}'
+                        .format(dataset_label=dataset_label)
+                    )
+
+                    datasets_updated = True
+                except botocore_clienterror as exception:
+                    print((
+                        "Failed with exception = {}"
+                        "\nUnable to update database dataset "
+                        "due to vendor api failure.".format(exception)
+                    ))
 
         return datasets_updated
 
@@ -208,7 +244,11 @@ class MaintenanceManager:
             for_update=True
         )
 
+        if solutions is None:
+            return solutions_updated
+
         for _, solution in enumerate(solutions):
+
             # The solution arn is required to continue maintenance for this solution
             arn = solution.get('arn', None)
 
@@ -232,29 +272,35 @@ class MaintenanceManager:
                     prefix=[client.name, engine_slug]
                 )
 
-                # train a new model if should_create_version True
-                new_solution = self.solution_state_manager.update_state(
-                    solution=solution,
-                    solution_base_name=new_solution_base_name
-                )
+                try:
+                    # train a new model if should_create_version True
+                    new_solution = self.solution_state_manager.update_state(
+                        solution=solution,
+                        solution_base_name=new_solution_base_name
+                    )
 
-                # get solution arn
-                solution_arn = new_solution.get('arn')
-                print(
-                    'Successfully started creation of solution version: {solution_version_arn}'
-                    .format(solution_version_arn=solution_arn)
-                )
+                    # get solution arn
+                    solution_arn = new_solution.get('arn')
+                    print(
+                        'Successfully started creation of solution version: {solution_version_arn}'
+                        .format(solution_version_arn=solution_arn)
+                    )
 
-                # update immedidately and change status
-                self.query_helper.update_solution(
-                    client_id=client.id,
-                    service_id=self.service.id,
-                    engine_slug=engine_slug,
-                    recipe=solution['recipe'],
-                    config=new_solution
-                )
+                    # update immedidately and change status
+                    self.query_helper.update_solution(
+                        client_id=client.id,
+                        service_id=self.service.id,
+                        engine_slug=engine_slug,
+                        recipe=solution['recipe'],
+                        config=new_solution
+                    )
 
-                solutions_updated = True
+                    solutions_updated = True
+                except botocore_clienterror as exception:
+                    print((
+                        "Failed with exception = {}"
+                        "\nUnable to update vendor solution.".format(exception)
+                    ))
 
             elif curr_state.get('should_refresh_version_status') is True:
                 # update state step >> solution manager
@@ -277,13 +323,16 @@ class MaintenanceManager:
                     service_id=self.service.id,
                     engine_slug=engine_slug,
                     recipe=solution['recipe'],
-                    config={'version_status': new_version_status}
+                    config={
+                        'version_status': new_version_status
+                    }
                 )
 
                 solutions_updated = True
 
         return solutions_updated
 
+    # pylint: disable=too-many-locals
     def perform_campaign_maintenance(
             self, *,
             client: dict,
@@ -312,6 +361,9 @@ class MaintenanceManager:
             for_update=True
         )
 
+        if campaigns is None:
+            return campaigns_updated
+
         for _, campaign in enumerate(campaigns):
             # The campaign arn is required to continue maintenance for this campaign
             arn = campaign.get('arn', None)
@@ -325,6 +377,7 @@ class MaintenanceManager:
                 service_id=self.service.id,
                 engine_slug=engine_slug,
                 from_cache=False,
+                for_update=True,
                 recipe=campaign['recipe']
             )
 
@@ -348,23 +401,29 @@ class MaintenanceManager:
                     )
                 )
 
-                new_campaign = self.campaign_state_manager.update_state(
-                    campaign=campaign,
-                    solution=solution
-                )
+                try:
+                    new_campaign = self.campaign_state_manager.update_state(
+                        campaign=campaign,
+                        solution=solution
+                    )
 
-                print('Successfully started updating campaign: {arn}'.format(arn=arn))
+                    print('Successfully started updating campaign: {arn}'.format(arn=arn))
 
-                # try and update else fail
-                self.query_helper.update_campaign(
-                    client_id=client.id,
-                    service_id=self.service.id,
-                    engine_slug=engine_slug,
-                    recipe=campaign['recipe'],
-                    config=new_campaign
-                )
+                    # try and update else fail
+                    self.query_helper.update_campaign(
+                        client_id=client.id,
+                        service_id=self.service.id,
+                        engine_slug=engine_slug,
+                        recipe=campaign['recipe'],
+                        config=new_campaign
+                    )
 
-                campaigns_updated = True
+                    campaigns_updated = True
+                except botocore_clienterror as exception:
+                    print((
+                        "Failed with exception = {}"
+                        "\nUnable to update vendor campaign.".format(exception)
+                    ))
 
             elif curr_state.get('should_refresh_status') is True:
                 # get current version arn
@@ -372,6 +431,7 @@ class MaintenanceManager:
                     'latest_solution_version_arn'
                 )
                 new_status = curr_state.get('new_status')
+                last_refreshed_at = curr_state.get('last_refreshed_at')
 
                 print(
                     'Refreshing status on campaign: {arn} for client: {client_name} to {status}'
@@ -382,13 +442,22 @@ class MaintenanceManager:
                     )
                 )
 
+                # solution refresh to account for endpoint roll-back
+                # when sagemaker endpoint update fails, it rolls back
+                # as such campaign in db and vendor will differ
+                # this is a roll back
+                refresher_solution = {
+                    'status':new_status,
+                    'solution_version_arn':latest_solution_version_arn,
+                    'last_updated_at': last_refreshed_at
+                }
                 # update status in db
                 self.query_helper.update_campaign(
                     client_id=client.id,
                     service_id=self.service.id,
                     engine_slug=engine_slug,
                     recipe=campaign['recipe'],
-                    config={'status':new_status}
+                    config=refresher_solution
                 )
 
                 campaigns_updated = True
