@@ -229,18 +229,19 @@ class SolutionStateManager:
         # update the arn with the new arn
         new_solution.update({
             'arn': new_solution_arn
-            })
+        })
 
         vendor_solution = self.sagemaker.describe_training_job(
             TrainingJobName=new_solution_arn
         )
 
         vendor_solution_version_status = self.get_vendor_version_status(
-            vendor_solution=vendor_solution)
+            vendor_solution=vendor_solution
+        )
         # default to inProgress
         new_solution.update({
             'version_status': vendor_solution_version_status
-            })
+        })
 
         new_solution.update({
             'version_last_created_at': datetime.strftime(
@@ -250,9 +251,13 @@ class SolutionStateManager:
         })
 
         next_version_creation_time = solution.get('next_version_creation_time', None)
-        if next_version_creation_time:
-            new_solution.update({
-                'next_version_creation_time': next_version_creation_time
-            })
 
+        if next_version_creation_time:
+            version_creation_frequency = solution['version_creation_frequency']
+            new_solution.update({
+                'next_version_creation_time': self.date_utils.get_next_time(
+                    this_time=next_version_creation_time,
+                    frequency=version_creation_frequency
+                )
+            })
         return new_solution
