@@ -29,15 +29,21 @@ class SagemakerEngineHelpers(EngineQueryHelper):
                 'recommend-prefix1-prefix2-prefix3-lookup'
         """
         # join prefixes
-        if len(prefix) > 0:
+        if prefix is not None:
             prefixes = "-".join(prefix)
 
-        # build cache key
-        cache_key = '{service}-{prefixes}-{group}'.format(
-            service=service,
-            prefixes=prefixes,
-            group=group
-        )
+            # build cache key
+            cache_key = '{service}-{prefixes}-{group}'.format(
+                service=service,
+                prefixes=prefixes,
+                group=group
+            )
+        else:
+            # build cache key
+            cache_key = '{service}-{group}'.format(
+                service=service,
+                group=group
+            )
 
         return cache_key.replace(" ", "-").lower()
 
@@ -110,6 +116,7 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id: int,
             engine: str,
             from_cache: bool = True,
+            for_update: bool = False,
             label: str,
             key: str
     ):
@@ -122,7 +129,7 @@ class SagemakerEngineHelpers(EngineQueryHelper):
                 engine (str): Subscription property engine
                 from_cache (bool): Retrieve the datasets from cache - Default True
                 label (str): Label describing what the dataset is for
-                key (str): Dataset ket to lookup
+                key (str): Dataset key to lookup
 
             returns:
                 result (dict): The result of the lookup
@@ -134,6 +141,7 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine,
             from_cache=from_cache,
+            for_update=for_update,
             **dataset_kwargs
         )
 
@@ -212,7 +220,8 @@ class SagemakerEngineHelpers(EngineQueryHelper):
         subscription = self.get_subscription(
             client_id=client_id,
             service_id=service_id,
-            from_cache=False
+            from_cache=False,
+            for_update=True
         )
 
         # get dataset index
@@ -221,6 +230,7 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine_slug,
             from_cache=from_cache,
+            for_update=False,
             **kwargs
         )
         # update config
@@ -232,7 +242,8 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine_slug,
             entity_key='datasets',
-            from_cache=False
+            from_cache=False,
+            for_update=False
         )
 
         # update entity
@@ -266,7 +277,8 @@ class SagemakerEngineHelpers(EngineQueryHelper):
         subscription = self.get_subscription(
             client_id=client_id,
             service_id=service_id,
-            from_cache=False
+            from_cache=False,
+            for_update=True
         )
 
         # get campaign index
@@ -275,6 +287,7 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine_slug,
             from_cache=from_cache,
+            for_update=False,
             **kwargs
         )
 
@@ -286,7 +299,8 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine_slug,
             entity_key='campaigns',
-            from_cache=False
+            from_cache=False,
+            for_update=False
         )
         # update the engine campaign
         self.update_engine_entity_on_index(
@@ -319,7 +333,8 @@ class SagemakerEngineHelpers(EngineQueryHelper):
         subscription = self.get_subscription(
             client_id=client_id,
             service_id=service_id,
-            from_cache=False
+            from_cache=False,
+            for_update=True
         )
 
         solution, solution_index = self.get_solution(
@@ -327,6 +342,7 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine_slug,
             from_cache=from_cache,
+            for_update=False,
             **kwargs
         )
         solution.update(config)
@@ -336,7 +352,8 @@ class SagemakerEngineHelpers(EngineQueryHelper):
             service_id=service_id,
             engine_slug=engine_slug,
             entity_key='solutions',
-            from_cache=False
+            from_cache=False,
+            for_update=False
         )
         # update the engine solution
         self.update_engine_entity_on_index(
